// (c) Theo Verelst
// Uncommercial use only permitted with this notice clearly 
// visible, commercial rights reserved, contact: theover@tiscali.nl

#include "Talkthrough.h"
#include <math.h>


//--------------------------------------------------------------------------//
// Function:	Process_Data()												//
//																			//
// Description: This function is called from inside the SPORT0 ISR every 	//
//				time a complete audio frame has been received. The new 		//
//				input samples can be found in the variables iChannel0LeftIn,//
//				iChannel0RightIn, iChannel1LeftIn and iChannel1RightIn 		//
//				respectively. The processed	data should be stored in 		//
//				iChannel0LeftOut, iChannel0RightOut, iChannel1LeftOut,		//
//				iChannel1RightOut, iChannel2LeftOut and	iChannel2RightOut	//
//				respectively.												//
//                                                                          //
//              The rest is added by Theo Verelst to implement a analog     //
//              sound synthesizer and chorus effect unit which is also      //
//              usable with an analog input signal in parallel.             //
//              On the blackfin 533 EZLITE board, this version renders 4    //
//              high quality oscillators with additive waveform samples     //
//              under MIDI control, with variable oscilator asignment,     //
//              ADSR envelope, and coupled cutoff frequency 4th order       //
//              digital (linear analog simulation) filter.                  //
//                                                                          //
//                                                                          //
//                                                                          //
//                                                                          //
//--------------------------------------------------------------------------//


#define SW7 0x0100        // switch SW7, LED pattern 0 (PF8)
#define SW6 0x0200        // switch SW6, LED pattern 1 (PF9)
#define SW4 0x0400        // switch SW4, LED pattern 2 (PF10)
#define SW5 0x0800        // switch SW5, stop (PF11)

int buttonsprev = 0, bupre1=1, bupre2 =2;
int debou=0;
unsigned char out=1;
#define MAXCHARDEF 2
// unsigned char dis57[5*MAXCHARDEF] = {0xf8, 0x24, 0x22, 0x24, 0xf8 , 0x82, 0x82, 0x92, 0x92, 0x7c };

unsigned char *midipointer, midicount;
unsigned char measurenext = 0;

extern int *sinewave;
extern int si1,si2,si3;
extern int sincr1, sincr2, sincr3;
extern int sc, vul, vur;

extern int *dell, *delr, dml, dmr, dil, dir, dol, dor, dom;

extern int sine[], sinincr[];
extern unsigned char midinote[], midivel[], midibyte;
extern char midistate, midipstate;
unsigned char mi1, mi2, mi3;

extern int *wavetable, wtlen;
extern int incrfromnote[];
extern char notestate[];
extern int noteampl[];
extern unsigned char attack, decay, sustain, release;
char transpose = 64, detune = 67;
unsigned char chorusspeed = 1, chorusdepth = 0, cutoff = 64, resonance = 0;
extern int fils[MAXPOLY][4], filc[MAXPOLY];

char *dmm[] = {"Attack", "Decay", "Sustain", "Release",
               "Chorus Freq.", "Chorus Depth", "Transpose", "Detune", 
               "Cutoff" ,"Resonance" };
char *dm;
unsigned short dmc = 0x0ff, knobspeed = 0;
int dodis =0, dodisval = 0;
unsigned char modulation = 0, value = 0, knob = 0;

void initmidi()
{
	midipointer = MIDIBASE;
}



//--------------------------------------------------------------------------//
// Function:	Process_Data()												//
//																			//
// Description: This function is called from inside the SPORT0 ISR every 	//
//				time a complete audio frame has been received. The new 		//
//				input samples can be found in the variables iChannel0LeftIn,//
//				iChannel0RightIn, iChannel1LeftIn and iChannel1RightIn 		//
//				respectively. The processed	data should be stored in 		//
//				iChannel0LeftOut, iChannel0RightOut, iChannel1LeftOut,		//
//				iChannel1RightOut, iChannel2LeftOut and	iChannel2RightOut	//
//				respectively.												//
//--------------------------------------------------------------------------//
void Process_Data(void)
{
	int t,v,w,i,j,j0,j1,j2;
	int k,m,t1,t2,cu;
	unsigned char ext_switches, l;
	
	k = *pFIO_FLAG_D;
	

	iChannel1LeftOut = iChannel0LeftIn;
	iChannel1RightOut = iChannel0RightIn;
	
    iChannel0LeftIn = iChannel0LeftIn >> (2+1);
    iChannel0RightIn = iChannel0RightIn >> (2+1);

    if (knobspeed <10000) knobspeed++;
    i = (* (unsigned char *) 0x20300000) & 0x0F;
   	switch (i-knob) {
   	 case 1:
   	 case -15:
   	     if (knobspeed > 1500) {
    	     if (value < 255) value++;
   	     } else {
       	     if (value < 245) value+=10;
       	     else
       	        value = 255;
    	 }
   	     knobspeed = 0;
   	  break;
   	 case -1:
   	 case 15:
   	     if (knobspeed > 1500) {
    	     if (value > 0) value--;
   	     } else {
       	     if (value > 10) value-=10;
       	     else
       	        value = 0;
    	 }
   	     knobspeed = 0;
   	  break;
   	 case 0:
   	 default:
   	  break;
   	}
   	knob = i;
    
    if (((* (unsigned char *) 0x20300006) & 0x20) == 0x20) {
    	if (((* (unsigned char *) 0x20300006) & 0xc0) != midicount) {
       		midicount = (*((unsigned char *) 0x20300006)) & 0xc0;
    		l = *midipointer = *((unsigned char *) 0x20300004);   /* Read byte */
    		midipointer++;
    		if (midipointer>= MIDIMAX+MIDIBASE) {
    			midipointer = MIDIBASE;
    		}
    		if (midistate == MIDINIT) { l = 0; midistate = MIDDONE; }
            if ((l & 0x80) != 0) { /* first message byte ? */
               midibyte = 1;
               switch (l>>4) {
               	case MIDNOTEON:
               	case MIDNOTEOFF:
               	case MIDCONT:
               	case MIDPITCH:
               	case MIDAFTERT:
               	  midistate = (l>>4);
               	  midipstate = midistate;
               	  break;
               	case MIDTIMING:
               	  break;
               	default:
               	  midistate = MIDUNKNOWN; /* Unknown message: wait for new status word */
               }
               mi1 = l;
            } else {
            	if (midistate == MIDDONE) {  /* More data without header: running status */
            		midibyte = 1;
                    midistate = midipstate;
            	}
            	   midibyte++;
            	   if (midistate != MIDUNKNOWN) {
            	      switch (midibyte) {
            	      	case 2: mi2 = l;
            	      	  break;
            	      	case 3: mi3 = l; midibyte = 0;
            	      	/* Here we received one of the 3 byte midi messages fully */
            	      	  switch (midistate) {
               	           case MIDNOTEON:
               	            if (mi3 == 0) {        /* Is actually note off ? */
                	            for (i=0; i<MAXPOLY; i++)
                	               if (midinote[i] == mi2) 
                	                  notestate[i] = NOTERELEASE;
               	            	      /* midinote[i] = 0; */
               	            } else {
               	            	unsigned char ma;
               	                   /*search for free generator */
                	            i=0; while (i<MAXPOLY && notestate[i] != NOTEOFF) i++;
                	            ma = 0;
                	            j = noteampl[0];
                	            if (i >= MAXPOLY) {
                                   i=0;
                                   while (i<MAXPOLY && notestate[i] != NOTEOFF) {
                                      i++;
                	                  if (noteampl[i] < j && (notestate[i] == NOTEDECAY 
                	                  || notestate[i] == NOTESUSTAIN
                	                  || notestate[i] == NOTERELEASE )) {
                	               	     j = noteampl[i];
                	               	     ma = i; 
                	                  }
                	               }
                	               i=ma; 
                	            }
                	            if (i< MAXPOLY) {
                	               midinote[i] = mi2;
                	               midivel[i] = mi3;
                	               sinincr[i] = incrfromnote[mi2+(transpose-64)];
                	               sine[i] = 0;   /* initial phase == 0 */
                	               if (notestate[i] == NOTEOFF)
                                      noteampl[i] = 0;
                                   notestate[i] = NOTEATTACK;
                	            }
                	            i=0; while (i<MAXPOLY && notestate[i] != NOTEOFF) i++;
                	            if (detune != 64 && i< MAXPOLY) {
                	               midinote[i] = mi2;
                	               midivel[i] = mi3;
                	               sinincr[i] = incrfromnote[mi2+(transpose-64)]+(detune-64);
                	               sine[i] = 0;   /* initial phase == 0 */
                	               notestate[i] = NOTEATTACK;
                	               noteampl[i] = 0;
                	            }

               	            }
               	            break;
               	           case MIDNOTEOFF:
               	            for (i=0; i<MAXPOLY; i++)
               	               if (midinote[i] == mi2) 
               	                  notestate[i] = NOTERELEASE;
           	            	      /* midinote[i] = 0; */
               	            break;
               	           case MIDCONT:
               	            if (mi2 == 1) {  /* mod wheel */
               	               if (dmc == 5) {
               	               	  value = mi3;
             	               }
           	                   chorusdepth = mi3;
                               sincr1 = (16000 * (64+ 6*mi3)/64);
                               sincr2 = (21023 * (64+ 6*mi3)/64);
                               sincr3 = ( 8111 * (64+ 6*mi3)/64);
               	            }
               	            if (mi2 == 123 && mi3 == 0) { /* all notes off */
                               for (i=0; i<MAXPOLY; i++)
           	                      notestate[i] = NOTERELEASE;
               	            }
               	            break;
               	           case MIDAFTERT:
               	            ;
               	            break;
               	           case MIDPITCH:
               	            value = sizeof(dmm);
               	            break;
            	      	  	
            	      	  	default: break;
            	      	  }
            	      	  midistate = MIDDONE; 
            	      	  break;
            	      	default: midibyte = 0; midistate = MIDDONE;
            	      }
            	   }
            }
/*    		if (l == 144) midinote = 69;
    		if (l == 128) midinote = 0; */
    	}
    }

   	v=0;
	for (i=0; i<MAXPOLY; i++) {
       w=0;
       sine[i] = (sine[i] + sinincr[i]) % wtlen;
       if (midinote[i] != 0 && notestate[i] != NOTEOFF) {
       	   switch (notestate[i]) {
       	   	case NOTEATTACK:
       	   	  if (noteampl[i] > 0xfffd) {     /* See if max level is reached */
       	   	  	// notestate[i] = NOTESUSTAIN;
       	   	  	notestate[i] = NOTEDECAY;
       	   	  	noteampl[i] = 0x10000;
       	   	  }
              w += (noteampl[i]>>4) * (((midivel[i] * wavetable[sine[i]])/(MAXPOLY*128))>>4);
              noteampl[i] = (int) 0x10000 - ((attack* (0x10000 - noteampl[i])>>8))+1;
              // noteampl[i] = noteampl[i]+1;
             break;
       	   	case NOTEDECAY:
       	   	  if (noteampl[i] < 100) {     /* lowest amplitude --> switch generator off */
       	   	  	notestate[i] = NOTEOFF;
       	   	  	midinote[i] = 0;
       	   	  }
              w += (noteampl[i]>>4) * (((midivel[i] * wavetable[sine[i]])/(MAXPOLY*128))>>4);
              noteampl[i] = (((int) ((16384-256+decay)* noteampl[i]))>>14) -1;
       	   	  if (noteampl[i] < ((0x10000>>8) * sustain) ) {     /* sustain level reached ? */
       	   	  	notestate[i] = NOTESUSTAIN;
       	   	  }
              // noteampl[i] = noteampl[i]-1;
             break;
       	    case NOTESUSTAIN:
              w += (noteampl[i]>>4) * (((midivel[i] * wavetable[sine[i]])/(MAXPOLY*128))>>4);
             break;
       	   	case NOTERELEASE:
       	   	  if (noteampl[i] < 100) {     /* lowest amplitude --> switch generator off */
       	   	  	notestate[i] = NOTEOFF;
       	   	  	midinote[i] = 0;
       	   	  }
              w += (noteampl[i]>>4) * (((midivel[i] * wavetable[sine[i]])/(MAXPOLY*128))>>4);
              noteampl[i] = (((int) ((16384-256+release)* noteampl[i]))>>14) -1;
              // noteampl[i] = (((int) ((16384-256+release-((* (unsigned char *) 0x20300000) & 0x0F))* noteampl[i]))>>14) -1;
              // noteampl[i] = (((int) ((16384-((* (unsigned char *) 0x20300000) & 0x0F))* noteampl[i]))>>14) -1;
              // noteampl[i] = noteampl[i]-1;
             break;
       	    default:
              w += ((midivel[i] * wavetable[sine[i]])/(MAXPOLY*128)) << 8;
       	   }
       };
       {
       	  int d;
       	  d = (resonance * fils[i][3]/16)/-4;
       	  d = (w/2+d-fils[i][0])>>8;
   	      cu = (cutoff*noteampl[i])>>15;
   	      if (cu > 255) cu = 255;
          fils[i][0] = fils[i][0] + cu * ((cu * d)>>8);
          for (j=1;j<4;j++) {
             d = (fils[i][j-1]-fils[i][j])>>8;
             fils[i][j] = fils[i][j] + cu * ((cu * d)>>8); /* + (sinincr[i]*d)/4; */
          }
/*       	  d = (fils1[i]-fils2[i])>>8;
          fils2[i] = fils2[i] + cutoff * ((cutoff * d)>>8);
       	  d = (fils2[i]-fils3[i])>>8;
          fils3[i] = fils3[i] + cutoff * ((cutoff * d)>>8);
*/
          //fils[i][1] += ((short) cutoff * (fils[i][0]-fils[i][1]))>>7;
          //fils[i][2] += ((int) cutoff * (fils[i][1]-fils[i][2]))>>7;
          //fils[i][3] += ((int) cutoff * (fils[i][2]-fils[i][3]))>>7;
          v+=fils[i][3];
          //v+=cutoff * (w>>5);
          //v+=w;
       }
	}

/*	iChannel0LeftOut += v;
	iChannel0RightOut += v;
	*/
	
	m = v;
	// iChannel0LeftIn += v/4;
	// iChannel0RightIn += v/4;
	iChannel0LeftIn += v/3;
	iChannel0RightIn += v/3;

    si1 = (si1 + sincr1) % (MAXSINEWAVE << 16);
    si2 = (si2 + sincr2) % (MAXSINEWAVE << 16);
    si3 = (si3 + sincr3) % (MAXSINEWAVE << 16);

    // i = (* (unsigned char *) 0x20300000) & 0x0F;
    i = chorusdepth;
    //((i*sinewave[si1>>16])>>(23-5+3));
    j0 = i*sinewave[si1>>16];
    j  = j0 >>(23-5+3);  /* 21 bit right shift, 11 (signed) bits left, of which 3 relevant */
    
    t = dil - (dol +  j);
    if (t < 0) t+= dml;
    if (j0<0) {
       t1 = t+1;
       j2 = (j0>>10) & 0x07FF;  /* Linear interpolation factors */
       j1 = 0x07FF - j2;
    } else {
       t1 = t-1;
       j1 = (j0>>10) & 0x07FF;  /* Linear interpolation factors */
       j2 = 0x07FF - j1;
    }
    if (t1 >= dml) t1-= dml;
    if (t1 < 0) t1+= dml;
	iChannel0LeftOut = ((j2*(dell[t]>>12)+j1*(dell[t1]>>12))<<2)+ iChannel0LeftIn;
	/* iChannel0LeftOut = dell[t] + iChannel0LeftIn; */
	dell[dil] = iChannel0LeftIn;
	dil = (dil+1) % dml;
	
	//	iChannel0RightOut = m/4;

    j0 = i*sinewave[si2>>16];
    j  = j0 >>(23-5+3);  /* 21 bit right shift, 11 (signed) bits left, of which 3 relevant */
    
    t = dir - (dor +  j);
    if (t < 0) t+= dmr;
    if (j0<0) {
       t1 = t+1;
       j2 = (j0>>10) & 0x07FF;  /* Linear interpolation factors */
       j1 = 0x07FF - j2;
    } else {
       t1 = t-1;
       j1 = (j0>>10) & 0x07FF;  /* Linear interpolation factors */
       j2 = 0x07FF - j1;
    }
    if (t1 >= dmr) t1-= dmr;
    if (t1 < 0) t1+= dmr;
	iChannel0RightOut = ((j2*(delr[t]>>12)+j1*(delr[t1]>>12))<<2)+ iChannel0RightIn;
	/* iChannel0LeftOut = dell[t] + iChannel0LeftIn; */
	delr[dir] = iChannel0RightIn;
	dir = (dir+1) % dmr;
	

    j0 = i*sinewave[si3>>16];
    j  = j0 >>(23-5+3);  /* 21 bit right shift, 11 (signed) bits left, of which 3 relevant */
    
    t = dil - (dom +  j);
    if (t < 0) t+= dml;
    if (j0<0) {
       t1 = t+1;
       j2 = (j0>>10) & 0x07FF;  /* Linear interpolation factors */
       j1 = 0x07FF - j2;
    } else {
       t1 = t-1;
       j1 = (j0>>10) & 0x07FF;  /* Linear interpolation factors */
       j2 = 0x07FF - j1;
    }
    if (t1 >= dml) t1-= dml;
    if (t1 < 0) t1+= dml;
	iChannel0LeftOut += ((j2*(dell[t]>>12)+j1*(dell[t1]>>12))<<2);
	iChannel0RightOut += ((j2*(dell[t]>>12)+j1*(dell[t1]>>12))<<2);
	/* iChannel0LeftOut = dell[t] + iChannel0LeftIn; */

	
/*    t = dir - (dor + ((i*sinewave[si2>>16])>>(23-5+3)) );
    if (t < 0) t+= dmr;
	iChannel0RightOut = delr[t] + iChannel0RightIn;
	delr[dir] = iChannel0RightIn;
	dir = (dir+1) % dmr;
	
	i = (dom + ((i*sinewave[si3>>16])>>(23-5+3)) );
    t = dil - i;
    if (t < 0) t+= dml;
    v = dell[t];
    t = dir - i;
    if (t < 0) t+= dmr;
    v += delr[t];
	iChannel0LeftOut += v;
	iChannel0RightOut += v;
*/
	
	iChannel1LeftOut = m;
	iChannel1RightOut = m;

	
	if (chorusdepth == 0) {
       iChannel0LeftOut = m;
       iChannel0RightOut = m;
	}
    //iChannel0LeftOut = -iChannel0LeftOut; /* For bridging the amp only */
	
	// The Bypass switches
/*	ext_switches = (* (unsigned char *) (0x20300006)) & 0x1f;
	if (ext_switches == 0) {
       iChannel0LeftOut = iChannel1LeftOut;
       iChannel0RightOut = iChannel1RightOut;
    }
*/	

    i = ((* (unsigned char *) 0x20300000) & 0x60) >> 5;  /* Get switch states in b0,b1 */

	if (  ((i&1) == 0 && debou == 0 && bupre1 == 1)
	   || ((i&2) == 0 && debou == 0 && bupre2 == 2) ) {
	   	if ((i&1) == 0) {
	   	   dmc++;
		   if (dmc >= (sizeof(dmm)/sizeof(dmm[0]))) dmc=0;
		   bupre1 = i&1;
	   	} else {
	   		if (dmc == 0 ) dmc = (sizeof(dmm)/sizeof(dmm[0]))-1;
	   		else
	   		   if (dmc >= (sizeof(dmm)/sizeof(dmm[0])))
	   		      dmc = (sizeof(dmm)/sizeof(dmm[0]))-1;
	   		   else
                  dmc--;
	   	    bupre2 = i&2;
	   	}
		dm = dmm[dmc];
		dodis =1;
		debou = 48*20;
	    switch (dmc) {
		 case 0:
		   value = attack;
		  break;
		 case 1:
		   value = decay;
		  break;
		 case 2:
		   value = sustain;
		  break;
		 case 3:
		   value = release;
		  break;
		 case 4:
		   value = chorusspeed;
		  break;
		 case 5:
		   value = chorusdepth;
		  break;
		 case 6:
		   value = transpose;
		  break;
		 case 7:
		   value = detune;
		  break;
		 case 8:
		   value = cutoff;
		  break;
		 case 9:
		   value = resonance;
		  break;
		 default:
		  break;
	}
	}
	if ((i&1) == 1 && debou == 0 && bupre1 == 0 ) {
		debou = 48*20;
		bupre1 = 1;
	}

/*	if ((i&2) == 0 && debou == 0 && bupre2 == 2 ) {
		dm = dmm[0];
		dmc = 0;
		dodis =1;
		debou = 48*20;
		bupre2 = i&2;
	}
*/
	if ((i&2) == 2 && debou == 0 && bupre2 == 0 ) {
		debou = 48*20;
		bupre2 = 2;
	}

	if (debou > 0) debou--;
	
	switch (dmc) {
		case 0:
		   attack = value;
		 break;
		case 1:
		   decay = value;
		 break;
		case 2:
		   sustain = value;
		 break;
		case 3:
		   release = value;
		 break;
		case 4:
           chorusspeed = value;
		 break;
		case 5:
		   chorusdepth = value;
		 break;
		case 6:
		   transpose = value;
		 break;
		case 7:
		   detune = value;
		 break;
		case 8:
		   cutoff = value;
         break;
		case 9:
           resonance = value;
		 break;
		default:
		 break;
	}
		
	sc = (sc +1) % 48000;

    if (!(buttonsprev & SW7))
	 if (k & SW7) {
       *pFlashA_PortB_Data = (unsigned char) 2;
	   out = out << 1;       
	   * (unsigned char *) (0x20300000) = out;
	}
    if (!(buttonsprev & SW6))
	 if (k & SW6) {
	   *pFlashA_PortB_Data = (unsigned char) 1;
	   out = 1;
	   * (unsigned char *) (0x20300000) = out;
	}
    if (!(buttonsprev & SW5))
	 if (k & SW5) {
	   out = * (unsigned char *) (0x20300000);
	   *pFlashA_PortB_Data = (unsigned char) out;
	}

	i = ((* (unsigned char *) 0x20300000) >> 4) & 0x07;
	j = (((* (unsigned char *) 0x20300000) >> 0) & 0x0F)/8;

	j = * (unsigned char *) (0x20300000);

    buttonsprev = k;
    
}


