// (c) Theo Verelst
// Uncommercial use only permitted with this notice clearly 
// visible, commercial rights reserved, contact: theover@tiscali.nl
// Based on the piecewise contained audio talkthrough from AD
// This project implements a 
//
//   Audio Synthesizer
// 
// in prototype form, including a chorus unit. 
// This is a musical instrument effect which adds richness to the sound 
// by mixing in various delayed components of the input signal 
// to the output, each with a varying delay lenght, modulated by 
// independent low frequency sine waves.
//
//--------------------------------------------------------------------------//
//																			//
//	 Name: 	Talkthrough for the ADSP-BF533 EZ-KIT Lite						//
//																			//
//--------------------------------------------------------------------------//
//																			//
//	(C) Copyright 2003 - Analog Devices, Inc.  All rights reserved.			//
//																			//
//	Project Name:	BF533 C Talkthrough TDM									//
//																			//
//	Date Modified:	04/03/03		HD		Rev 1.0							//
//																			//
//	Software:		VisualDSP++3.1											//
//																			//
//	Hardware:		ADSP-BF533 EZ-KIT Board									//
//																			//
//	Connections:	Connect an input source (such as a radio) to the Audio	//
//					input jack and an output source (such as headphones) to //
//					the Audio output jack									//
//																			//
//	Purpose:		This program sets up the SPI port on the ADSP-BF533 to  //
//					configure the AD1836 codec.  The SPI port is disabled 	//
//					after initialization.  The data to/from the codec are 	//
//					transfered over SPORT0 in TDM mode						//
//																			//
//--------------------------------------------------------------------------//

#include "Talkthrough.h"
#include "sysreg.h"
#include "ccblkfn.h"
#include <math.h>

//--------------------------------------------------------------------------//
// Variables																//
//																			//
// Description:	The variables iChannelxLeftIn and iChannelxRightIn contain 	//
//				the data coming from the codec AD1836.  The (processed)		//
//				playback data are written into the variables 				//
//				iChannelxLeftOut and iChannelxRightOut respectively, which 	//
//				are then sent back to the codec in the SPORT0 ISR.  		//
//				The values in the array iCodec1836TxRegs can be modified to //
//				set up the codec in different configurations according to   //
//				the AD1885 data sheet.										//
//--------------------------------------------------------------------------//
// left input data from ad1836
int iChannel0LeftIn, iChannel1LeftIn;
// right input data from ad1836
int iChannel0RightIn, iChannel1RightIn;
// left ouput data for ad1836	
int iChannel0LeftOut, iChannel1LeftOut;
// right ouput data for ad1836
int iChannel0RightOut, iChannel1RightOut;
// array for registers to configure the ad1836
// names are defined in "Talkthrough.h"
volatile short sCodec1836TxRegs[CODEC_1836_REGS_LENGTH] =
{									
					DAC_CONTROL_1	| 0x000,
					DAC_CONTROL_2	| 0x000,
					DAC_VOLUME_0	| 0x3ff,
					DAC_VOLUME_1	| 0x3ff,
					DAC_VOLUME_2	| 0x3ff,
					DAC_VOLUME_3	| 0x3ff,
					DAC_VOLUME_4	| 0x3ff,
					DAC_VOLUME_5	| 0x3ff,
					ADC_CONTROL_1	| 0x000,
					ADC_CONTROL_2	| 0x180,
					ADC_CONTROL_3	| 0x000
					
};
// SPORT0 DMA transmit buffer
volatile int iTxBuffer1[8];
// SPORT0 DMA receive buffer
volatile int iRxBuffer1[8];

extern Initaudiobuffer();
extern initmidi();
extern char *dm;
extern int dodis, dodisval;
extern unsigned char modulation, value;

char str[64] = "Line 2";

int *sinewave;
int sc, vul, vur;
int si1,si2,si3;
int sincr1, sincr2, sincr3;
int *dell, *delr, dml, dmr, dil, dir, dol, dor, dom;


int sine[MAXPOLY], sinincr[MAXPOLY];
unsigned char midinote[MAXPOLY], midivel[MAXPOLY], midibyte;
int incrfromnote[127];
char notestate[MAXPOLY];
int noteampl[MAXPOLY];
unsigned char attack = 244, decay = 254, sustain = 128, release = 253;
char midistate, midipstate;
int *wavetable, wtlen;
//int fils[MAXPOLY][4], filc[MAXPOLY];
int fils[MAXPOLY][4], filc[MAXPOLY];

initdelay()
{
	int i;
	
	dell =   1024*1024; dml = 48000;    // Delay Line Left, 1MB, max set to 1 sec
	delr = 2*1024*1024; dmr = 48000;    // Delay Line Right, base address, # regarded samples (fits in about 200kB)
	dil=0; dir=0;                       // Delay Index Left and Right
	dol =   200; dor =   200;           // Delay current Offset Left, Right
	dom = 200;                          // Delay offset mid channel
	
	for (i=0; i<dml; i++) dell[i] = 0;  // Zero/clear the lines
	for (i=0; i<dmr; i++) delr[i] = 0;
}

initsinewave() {
	int i,a;
	
	sinewave = (int *) 0;
	a = (int) pow(2,23) -1;
	
	for (i=0; i<MAXSINEWAVE;i++) {
		sinewave[i] = (int) (a*sin( 2.0*3.1415926535*((double) i)/((double) MAXSINEWAVE) ));
	}
	
	sc = 0; vul = 8; vur = 8;
//	ledvu(vul,vur);
	si1=0; si2=0;
//	sincr1 = 100<<16;
//	sincr1 = (3<<14);
//	sincr2 = (5<<13);
//	sincr3 = (1<<14);
	sincr1 = (16000);
	sincr2 = (21023);
	sincr3 = (8111);
}

initwavetable()
{
	int i,j,a;

	wavetable = (int *) WAVETABLE;
	a = (int) (pow(2,23) -1)/2;
	
	wtlen = 1024*128;
// ### Only for debugging !!!! ###
//#define SLOW
#ifdef SLOW
	for (i=0; i<wtlen;i++) {
        wavetable[i] = 0;
	   for (j=0; j<32; j++)
        wavetable[i] +=  (int) (1.0/j   * a*sin(   j *    2.0 *3.1415926535*((double) i)/((double) wtlen) ));
/*        wavetable[i] =  (int) (1.0   * a*sin(       2.0 *3.1415926535*((double) i)/((double) wtlen) ));
		wavetable[i] += (int) (1.0/2 * a*sin( 2.0 * 2.0 *3.1415926535*((double) i)/((double) wtlen) ));
		wavetable[i] += (int) (1.0/3 * a*sin( 3.0 * 2.0 *3.1415926535*((double) i)/((double) wtlen) ));
		wavetable[i] += (int) (1.0/4 * a*sin( 4.0 * 2.0 *3.1415926535*((double) i)/((double) wtlen) ));
		wavetable[i] += (int) (1.0/5 * a*sin( 5.0 * 2.0 *3.1415926535*((double) i)/((double) wtlen) ));
		wavetable[i] += (int) (1.0/6 * a*sin( 6.0 * 2.0 *3.1415926535*((double) i)/((double) wtlen) ));
		wavetable[i] += (int) (1.0/7 * a*sin( 7.0 * 2.0 *3.1415926535*((double) i)/((double) wtlen) ));
		wavetable[i] += (int) (1.0/8 * a*sin( 8.0 * 2.0 *3.1415926535*((double) i)/((double) wtlen) ));
		wavetable[i] += (int) (1.0/9 * a*sin( 9.0 * 2.0 *3.1415926535*((double) i)/((double) wtlen) ));
		wavetable[i] += (int) (1.0/10 * a*sin( 10.0 * 2.0 *3.1415926535*((double) i)/((double) wtlen) ));
		wavetable[i] += (int) (1.0/11 * a*sin( 11.0 * 2.0 *3.1415926535*((double) i)/((double) wtlen) ));
		wavetable[i] += (int) (1.0/12 * a*sin( 12.0 * 2.0 *3.1415926535*((double) i)/((double) wtlen) ));
		wavetable[i] += (int) (1.0/13 * a*sin( 13.0 * 2.0 *3.1415926535*((double) i)/((double) wtlen) ));
		wavetable[i] += (int) (1.0/14 * a*sin( 14.0 * 2.0 *3.1415926535*((double) i)/((double) wtlen) ));
		wavetable[i] += (int) (1.0/15 * a*sin( 15.0 * 2.0 *3.1415926535*((double) i)/((double) wtlen) ));
		wavetable[i] += (int) (1.0/16 * a*sin( 16.0 * 2.0 *3.1415926535*((double) i)/((double) wtlen) ));
*/
	}
#else
	for (i=0; i<wtlen;i++)
        wavetable[i] = *(((int *) 0x20100000)+i);

#endif
}

initsynth() {
	int i,j;
	
	midistate = MIDINIT;
	midibyte = 0;
	initwavetable();
	
	for (i=0; i< MAXPOLY; i++) {
	   sine[i] = 0;
	   sinincr[i] = 101;
	   midinote[i] = 0;
	   midivel[i] = 64;
	   notestate[i] = NOTEOFF;
	   for (j=0; j<4; j++) {
          fils[i][j]=0;
	   }
       filc[i] = 0.1*0x0ffff;
	}
	
	for (i=0; i<127; i++)
	   incrfromnote[i] = (int) ( ((wtlen*440)/48000) * pow( 2, (i-69.0)/12));

}

discontrol(c)
char c;
{
       * (unsigned char *) (0x20300004) = 0;
       sc =0; while (sc<20) ;
       * (unsigned char *) (0x20300000) = c;
	   sc =0; while (sc<20) ;
       * (unsigned char *) (0x20300004) = 2;
	   sc =0; while (sc<20) ;
       * (unsigned char *) (0x20300004) = 0;
       sc =0; while (sc<20) ;
}

dismess(m)
char m[];
{
	int i;
    i=0;
    * (unsigned char *) (0x20300004) = 1;
    sc =0; while (sc<2) ;
    while ((i<100) && m[i] != 0x00) {
       * (unsigned char *) (0x20300000) = m[i];
       sc =0; while (sc<2) ;
       * (unsigned char *) (0x20300004) = 3;
       sc =0; while (sc<2) ;
       * (unsigned char *) (0x20300004) = 1;
       i++;
    }
    sc =0; while (sc<2) ;
}

initdis()
{
	unsigned char d[] = {0x38,0x3c,0x01,0x06,0x0c,0x02,0x80,0xff};
	unsigned char t1[] = " - The Source -";
	int i;
	
    * (unsigned char *) (0x20300000) = 0x30;
	sc =0; while (sc<(2300)) ;
    * (unsigned char *) (0x20300004) = 2;
	sc =0; while (sc<2300) ;
    * (unsigned char *) (0x20300004) = 0;

	sc =0; while (sc<(2300)) ;
    * (unsigned char *) (0x20300004) = 2;
	sc =0; while (sc<2300) ;
    * (unsigned char *) (0x20300004) = 0;

	sc =0; while (sc<2300) ;
    * (unsigned char *) (0x20300004) = 2;
	sc =0; while (sc<2300) ;
    * (unsigned char *) (0x20300004) = 0;

    i=0;
    while ((i<100) && d[i] != 0xff) {
    	discontrol(d[i]);
/*       * (unsigned char *) (0x20300000) = d[i];
	   sc =0; while (sc<20) ;
       * (unsigned char *) (0x20300004) = 2;
	   sc =0; while (sc<20) ;
       * (unsigned char *) (0x20300004) = 0;
*/
       i++;
    }
    sc =0; while (sc<20) ;
    * (unsigned char *) (0x20300004) = 1;

    t1[0] = 0x7f;
    t1[15] = 0x7e;
    dismess(t1);

/*
    i=0;
    while ((i<100) && t1[i] != 0x00) {
       * (unsigned char *) (0x20300000) = t1[i];
       sc =0; while (sc<2) ;
       * (unsigned char *) (0x20300004) = 3;
       sc =0; while (sc<2) ;
       * (unsigned char *) (0x20300004) = 1;
       i++;
    }
*/
}

//--------------------------------------------------------------------------//
// Function:	main														//
//																			//
// Description:	After calling a few initalization routines, main() just 	//
//				waits in a loop forever.  The code to process the incoming  //
//				data can be placed in the function Process_Data() in the 	//
//				file "Process_Data.c".										//
//--------------------------------------------------------------------------//
void main(void)
{
    short sh1,sh2,sh3;
	sysreg_write(reg_SYSCFG, 0x32);		//Initialize System Configuration Register
	Init_EBIU();
	Init_Flash();

	Enable_Sdram();
//    Initaudiobuffer();
    initdelay();
    initsinewave();
    initmidi();
    initsynth();

	Init1836();
	Init_Sport0();
	Init_DMA();
	Init_Sport_Interrupts();
	Enable_DMA_Sport0();
//	printf("%d\n",-102%10);
	
    initdis();
    while(1) {
    	if (dodis > 0) {
           discontrol(0x01);
           discontrol(0x02);
    	   dismess(dm);
           discontrol(0xC0);
    	   //sprintf(str,"%d",(int) (* (unsigned char *) (0x20300000))  );
    	   dismess("value: ");
    	   dodis = 0;
    	   dodisval = 1;
    	}
    	if (value != sh3 || dodisval > 0) {
           // discontrol(0xCD);
           discontrol(0xC7);
           sh3 = value;
    	   sh1 = sh3 /100;
    	   if (sh1 != 0)
    	      str[0] = ((unsigned char) sh1) + 48;
    	   else
    	      str[0] = ' ';
    	   sh2 = (value - (sh1*100))/10;
    	   if (sh2 != 0 || str[0] != ' ')
    	      str[1] = ((unsigned char) sh2) + 48;
    	   else
    	      str[1] = ' ';
           str[2] = ((unsigned char) (value %10)) + 48;
           str[3] = '\0';
    	   dismess(str);
    	   dodisval = 0;
    	}
    };
}
